package com.arubanetworks.meridiansamples;

import static com.arubanetworks.meridiansamples.LocationSharingMapFragment.getFormattedAgeString;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.PointF;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.webkit.URLUtil;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.arubanetworks.meridian.editor.EditorKey;
import com.arubanetworks.meridian.internal.util.Dev;
import com.arubanetworks.meridian.maps.MapFragment;
import com.arubanetworks.meridian.maps.Marker;
import com.arubanetworks.meridian.maps.Transaction;
import com.arubanetworks.meridian.tags.TagBeacon;
import com.arubanetworks.meridian.tags.TagMarker;
import com.arubanetworks.meridian.tags.TagStream;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.bitmap.CircleCrop;
import com.bumptech.glide.request.target.CustomTarget;
import com.bumptech.glide.request.transition.Transition;

import java.util.HashMap;
import java.util.List;


/**
 * A Fragment to demonstrate the use of the TagStream for use in displaying Tags.  Demonstrates how to
 * show all tags for a map or a single tag in a location
 */
public class TagsFragment extends MapFragment implements TagStream.Listener {
    private static final String TAG_MAC = "meridian.TAG_MAC";
    // A collection to maintain our TagMarkers
    private final HashMap<TagBeacon, SamplesTagMarker> tagMarkers = new HashMap<>();

    // The TagStream itself
    private TagStream tagStream;
    private String tagMac;
    private SamplesTagMarker selectedTagMarker;

    public static TagsFragment newInstance(EditorKey mapKey) {
        TagsFragment f = new TagsFragment();
        f.setArguments(new MapFragment.Builder().setMapKey(mapKey).build().getArguments());
        return f;
    }

    public static TagsFragment newInstance(EditorKey mapKey, String tagMac) {
        TagsFragment f = new TagsFragment();
        Bundle args = new MapFragment.Builder().setMapKey(mapKey).build().getArguments();
        if (args != null) {
            args.putString(TAG_MAC, tagMac);
            f.setArguments(args);
        }
        return f;
    }

    @Override
    public void onMapLoadStart() {
        super.onMapLoadStart();
        // Close out any existing tags when loading a new map key
        if (tagStream != null) tagStream.stopUpdatingTags();

        // Clear any references to old markers so we don't re-add them to the new map on accident
        tagMarkers.clear();

        // Create our TagStream and start it.
        if (tagMac != null) {
            // only listen for specified tag
            tagStream = new TagStream.Builder()
                    .addTagMac(Application.APP_KEY, tagMac)
                    .setListener(this)
                    .build();
        } else {
            tagStream = new TagStream.Builder()
                    .addMapKey(getMapView().getMapKey())
                    .setListener(this)
                    .build();
        }
    }

    @Override
    public boolean onMarkerSelect(Marker marker) {
        selectedTagMarker = null;
        if (marker instanceof SamplesTagMarker && tagMarkers.containsValue(marker)){
            selectedTagMarker = ((SamplesTagMarker) marker);
        }
        return false;
    }

    @Override
    public boolean onMarkerDeselect(Marker marker) {
        if (marker instanceof SamplesTagMarker){
            selectedTagMarker = null;
        }
        return false;
    }

    @Override
    public Marker markerForSelectedMarker(Marker marker) {
        if (getContext() != null) {
            if (marker instanceof SamplesTagMarker) {
                SamplesTagMarker newMarker = new SamplesTagMarker(getContext(), ((SamplesTagMarker) marker).tag, true);
                newMarker.setPosition(marker.getPosition()[0],marker.getPosition()[1]);
                // update last heard field (in marker details)
                long ageInMillis = System.currentTimeMillis() - ((SamplesTagMarker) marker).tag.getLocation().getTimestamp().getTime();
                if (getContext() != null) {
                    selectedTagMarker.setDetails(getString(R.string.last_heard) + ": " + getFormattedAgeString(getContext().getResources(), ageInMillis));
                }
                selectedTagMarker = newMarker;
                return newMarker;
            }
        }
        return null;
    }

    @Override
    public void onTagsUpdated(List<TagBeacon> tags) {

        Transaction.Builder transaction = new Transaction.Builder().setType(Transaction.Type.UPDATE);

        // Iterate through the new or updated tags and add them to the map.
        for (TagBeacon tag : tags) {
            // ignore tags that are not on the current map
            if (getMapView() != null && getMapView().getMapKey() != null && !tag.getMapId().equals(getMapView().getMapKey().getId())) continue;
            SamplesTagMarker currentMarker = tagMarkers.get(tag);

            // check for a new Tag and add
            if (currentMarker == null) {
                if (getContext() == null) {return;}
                currentMarker = new SamplesTagMarker(getContext(), tag, false);
                tagMarkers.put(tag, currentMarker);
            } else {
                // Otherwise update our current marker to reflect the new tag state
                PointF point = tag.getLocation().getPoint();
                currentMarker.setPosition(point.x, point.y);
                if (selectedTagMarker != null && selectedTagMarker.tag.getMac().equalsIgnoreCase(tag.getMac())) {
                    selectedTagMarker.setPosition(tag.getLocation().getPoint().x,tag.getLocation().getPoint().y);
                    transaction.addMarker(selectedTagMarker).build();
                }
            }
            transaction.addMarker(currentMarker);
        }

        // Commit the transaction
        if (transaction.getSize() > 0)
            getMapView().commitTransaction(transaction.build());
    }

    @Override
    public void onTagsRemoved(List<TagBeacon> tags) {
        Transaction.Builder transaction = new Transaction.Builder().setType(Transaction.Type.REMOVE);

        for (TagBeacon tag : tags) {
            SamplesTagMarker currentMarker = tagMarkers.get(tag);

            // We can ignore removing the tags marker if we never added it.
            if (currentMarker == null) continue;

            transaction.addMarker(currentMarker);
        }
        getMapView().commitTransaction(transaction.build());
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            tagMac = getArguments().getString(TAG_MAC);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        // If we are paused we stop listening to tag updates
        if (tagStream != null) tagStream.stopUpdatingTags();
    }

    @Override
    public void onResume() {
        super.onResume();
        // If we are resuming, start listening for tags again.
        if (tagStream != null) tagStream.startUpdatingTags();

        /*// example how to scan for tags that are nearby.
        TagScan ts = new TagScan();
        ts.findNearbyTags(
                Application.APP_KEY,
                Application.TAG_MAC, // return specified tag or all tags if null
                0, // use default scan length (12s)
                0, // don't filter by RSSI
                new MeridianRequest.Listener<List<TagBeacon>>() {
                    @Override
                    public void onResponse(List<TagBeacon> response) {
                        if (response != null) {
                            Log.d("TagsFragment", "Found " + response.size() + " nearby tags");
                            for (TagBeacon tag : response) {
                                Log.d("TagsFragment", "Found:" + tag.toString());
                            }
                        }
                    }
                },
                null
        );*/
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // Clean up memory.
        if (tagStream != null) tagStream.dispose();
    }

    private class SamplesTagMarker extends TagMarker {

        private static final float SIZE_AVATAR_DP = 40.0f;

        private final TagBeacon tag;
        private Bitmap bitmap;
        private final boolean selected;  // if the tag marker is selected, this could be used to change the bitmap size for example

        SamplesTagMarker(Context context, TagBeacon tag, boolean selected) {
            super(context, tag);
            this.tag = tag;
            Context context1 = context.getApplicationContext();
            this.selected = selected;
            setWeight(1.1f);
            setName(tag.getName());
            long ageInMillis = System.currentTimeMillis() - tag.getLocation().getTimestamp().getTime();
            if (context1 != null) {
                setDetails(getString(R.string.last_heard) + ": " + getFormattedAgeString(context1.getResources(), ageInMillis));
            }
        }

        @Override
        public boolean getCollision() {
            return false;
        }

        @Override
        public boolean canBeSelected() {
            return !selected;
        }

        @Override
        public Bitmap getBitmap() {
            if (bitmap != null) {
                return bitmap;
            }

            if (getActivity() != null) {
                int size = Dev.get().dpToPix(selected ? SIZE_AVATAR_DP * 1.5f : SIZE_AVATAR_DP);
                if (URLUtil.isValidUrl(tag.getImageURL())) {
                    getActivity().runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Glide.with(getActivity().getApplicationContext())
                                    .asBitmap()
                                    .load(tag.getImageURL())
                                    .transform(new CircleCrop())
                                    .into(new CustomTarget<Bitmap>(size, size) {
                                        @Override
                                        public void onResourceReady(@NonNull Bitmap bitmap, @Nullable Transition<? super Bitmap> transition) {
                                            TagsFragment.SamplesTagMarker.this.bitmap = bitmap;
                                            TagsFragment.SamplesTagMarker.this.invalidate(true);
                                        }

                                        @Override
                                        public void onLoadCleared(@Nullable Drawable placeholder) {
                                            //do nothing
                                        }
                                    });
                        }
                    });
                } else {
                    getActivity().runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Bitmap bitmap = BitmapFactory.decodeResource(getResources(), R.drawable.mr_ic_tag);
                            SamplesTagMarker.this.bitmap = Bitmap.createScaledBitmap(bitmap, size, size, true);
                            SamplesTagMarker.this.invalidate(true);
                        }
                    });
                }
            }
            // still loading
            return null;
        }
    }
}
