package com.arubanetworks.meridiansamples

import android.Manifest
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.graphics.Color
import android.graphics.Matrix
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.ListView
import androidx.appcompat.app.ActionBarDrawerToggle
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.view.GravityCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.fragment.app.Fragment
import com.android.volley.VolleyError
import com.arubanetworks.meridian.location.LocationRequest
import com.arubanetworks.meridian.location.MeridianLocation
import com.arubanetworks.meridian.location.MeridianOrientation
import com.arubanetworks.meridian.maps.*
import com.arubanetworks.meridian.maps.directions.DirectionsDestination
import com.arubanetworks.meridian.maps.directions.DirectionsOptions
import com.arubanetworks.meridian.util.Strings
import com.arubanetworks.meridiansamples.Application.Companion.APP_KEY
import com.arubanetworks.meridiansamples.Application.Companion.MAP_KEY
import com.arubanetworks.meridiansamples.Application.Companion.PLACEMARK_UID
import org.acra.ACRA

class MainActivity : AppCompatActivity() {

    private var drawerLayout: DrawerLayout? = null
    private var drawerList: ListView? = null
    private var drawerToggle: ActionBarDrawerToggle? = null

    private var drawerTitle: CharSequence? = null
    private var actionBarTitle: CharSequence? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        drawerTitle = title
        actionBarTitle = drawerTitle
        drawerLayout = findViewById(R.id.drawer_layout)
        drawerList = findViewById(R.id.left_drawer)

        // set a custom shadow that overlays the main content when the drawer opens
        drawerLayout?.setDrawerShadow(R.drawable.drawer_shadow, GravityCompat.START)
        // set up the drawer's list view with items and click listener
        drawerList?.adapter = ArrayAdapter(this, R.layout.drawer_list_item, resources.getStringArray(R.array.section_titles))
        drawerList?.onItemClickListener = DrawerItemClickListener()

        // enable ActionBar app icon to behave as action to toggle nav drawer
        val actionBar = supportActionBar
        actionBar?.setDisplayHomeAsUpEnabled(true)
        actionBar?.setHomeButtonEnabled(true)

        // ActionBarDrawerToggle ties together the the proper interactions
        // between the sliding drawer and the action bar app icon
        drawerToggle = object : ActionBarDrawerToggle(
                this, /* host Activity */
                drawerLayout, /* DrawerLayout object */
                R.string.navigation_drawer_open, /* "open drawer" description for accessibility */
                R.string.navigation_drawer_close  /* "close drawer" description for accessibility */
        ) {
            override fun onDrawerClosed(view: View) {
                supportActionBar?.title = actionBarTitle
                delegate.invalidateOptionsMenu() // creates call to onPrepareOptionsMenu()
            }

            override fun onDrawerOpened(drawerView: View) {
                supportActionBar?.title = drawerTitle
                delegate.invalidateOptionsMenu() // creates call to onPrepareOptionsMenu()
            }
        }
        drawerLayout?.addDrawerListener(drawerToggle as ActionBarDrawerToggle)

        if (savedInstanceState == null) {
            if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                var permissions = arrayOf(Manifest.permission.ACCESS_FINE_LOCATION)
                if (Build.VERSION.SDK_INT == Build.VERSION_CODES.Q) {
                    permissions = arrayOf(Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_BACKGROUND_LOCATION)
                } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                    permissions = arrayOf(Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.BLUETOOTH_SCAN)
                }
                ActivityCompat.requestPermissions(this, permissions, 1)
            } else
                selectItem(0)
        }
        // preload the Google map to avoid a 3 second delay when switch to the demo
        if (useGoogleMaps()) {
            this.runOnUiThread {
                val mapView = com.google.android.gms.maps.MapView(this@MainActivity)
                mapView.onCreate(null)
            }
        }

        // check if we can load the deeplink and open associated placemark
        if (intent != null && intent.data != null) {
            if (checkUriAndOpenPlacemark(intent)) {
                drawerList?.let { drawerLayout?.closeDrawer(it) }
                intent.data = null // reset the intent, so we don't open this every single time
                return
            }
        }
    }

    private fun checkUriAndOpenPlacemark(intent: Intent?): Boolean {
        if (intent?.data == null || Strings.isNullOrEmpty(intent.dataString)) {
            return false
        }

        val data: Uri = intent.data!!
        val path: String? = data.path
        if (!Strings.isNullOrEmpty(path)) {
            val pathPieces: Array<String?> = path!!.split("/").toTypedArray()
            for (i in pathPieces.indices) {
                // try to open a placemark from deeplink
                if ("placemarks" == pathPieces[i] && i + 1 < pathPieces.size && pathPieces[i + 1] != null) {
                    val placemarkId = pathPieces[i + 1]
                    val mapFragment: MapFragment = MapFragment.Builder()
                        .setPlacemarkId(placemarkId)
                        .setMapKey(MAP_KEY)
                        .setAppKey(APP_KEY)
                        .build()
                    mapFragment.setMapEventListener(object : MapView.MapEventListener {
                        override fun onMapLoadStart() {}
                        override fun onMapLoadFinish() {}
                        override fun onPlacemarksLoadFinish() {
                            // example: highlight the placemark from deeplink and display callout
                            val mapView: MapView = mapFragment.mapView
                            if (mapView.allMarkers != null) {
                                for (marker in mapView.allMarkers) {
                                    mapView.onMarkerClick(marker)
                                    break
                                }
                            }
                        }

                        override fun onMapRenderFinish() {}
                        override fun onMapLoadFail(tr: Throwable) {}
                        override fun onMapTransformChange(transform: Matrix) {}
                        override fun onLocationUpdated(location: MeridianLocation) {}
                        override fun onOrientationUpdated(orientation: MeridianOrientation) {}
                    })
                    supportFragmentManager
                        .beginTransaction()
                        .replace(R.id.container, mapFragment)
                        .commit()
                    return true
                }
            }
        }
        // didn't find anything to open
        return false
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    /* Called whenever we call invalidateOptionsMenu() */
    override fun onPrepareOptionsMenu(menu: Menu): Boolean {
        // If the nav drawer is open, hide action items related to the content view
        val drawerOpen = drawerList?.let { drawerLayout?.isDrawerOpen(it) }
        menu.findItem(R.id.action_settings).isVisible = drawerOpen != true
        return super.onPrepareOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {

        // The action bar home/up action should open or close the drawer.
        // ActionBarDrawerToggle will take care of this.
        if (drawerToggle?.onOptionsItemSelected(item) == true) {
            return true
        }

        // Handle action buttons
        when (item.itemId) {
            R.id.action_settings -> {
                supportFragmentManager
                        .beginTransaction()
                        .replace(R.id.container, SettingsFragment())
                        .addToBackStack(null)
                        .commit()
                title = getString(R.string.action_settings)
                return true
            }
            R.id.action_manual_report -> {
                ACRA.getErrorReporter().handleException(null)
                return true
            }
            else -> return super.onOptionsItemSelected(item)
        }
    }

    private fun useGoogleMaps() : Boolean {
        // verify that the Google maps API key has been set in the manifest
        val app = this.packageManager.getApplicationInfo(this.packageName, PackageManager.GET_META_DATA)
        val apiKey = app.metaData.getString("com.google.android.geo.API_KEY")
        return (!Strings.isNullOrEmpty(apiKey) && apiKey != getString(R.string.add_google_maps_key))
    }

    /* The click listener for ListView in the navigation drawer */
    private inner class DrawerItemClickListener : AdapterView.OnItemClickListener {
        override fun onItemClick(parent: AdapterView<*>, view: View, position: Int, id: Long) {
            selectItem(position)
        }
    }

    private fun selectItem(position: Int) {

        // update the main content by replacing fragments
        val fragment: Fragment

        when (position) {
            0 -> {
                val builder = MapFragment.Builder()
                        .setMapKey(MAP_KEY)
                val mapOptions = MapOptions.getDefaultOptions()
                mapOptions.HIDE_OVERVIEW_BUTTON = true
                // example: how to set placemark markers text size
                //mapOptions.setTextSize(14)
                builder.setMapOptions(mapOptions)

                // examples: how to set directions options.
                /*val directionsOptions = DirectionsOptions.getDefaultOptions()
                directionsOptions.DIRECTIONS_CALLOUT_COLOR = ContextCompat.getColor(this, R.color.mr_color_primary)
                directionsOptions.DIRECTIONS_SLIDE_COLOR = ContextCompat.getColor(this, R.color.mr_color_primary)
                directionsOptions.ROUTE_OVERVIEW_BUTTON_COLOR = ContextCompat.getColor(this, R.color.mr_md_gray_600)
                directionsOptions.END_ROUTE_BUTTON_COLOR = ContextCompat.getColor(this, R.color.mr_md_gray_600)
                directionsOptions.ROUTE_OVERVIEW_BUTTON_TEXT_COLOR = ContextCompat.getColor(this, R.color.mr_white)
                directionsOptions.END_ROUTE_BUTTON_TEXT_COLOR = ContextCompat.getColor(this, R.color.mr_color_primary)
                directionsOptions.REORIENTATION_BANNER_COLOR = ContextCompat.getColor(this, R.color.mr_md_gray_600)
                directionsOptions.DIRECTIONS_INSTRUCTIONS_ON_TOP = true
                directionsOptions.DIRECTIONS_INSTRUCTIONS_TEXT_COLOR = Color.GRAY
                builder.setDirectionsOptions(directionsOptions)*/

                val mapFragment = builder.build()
                mapFragment.setMapEventListener(object : MapView.MapEventListener {

                    override fun onMapLoadFinish() { }

                    override fun onMapLoadStart() { }

                    override fun onPlacemarksLoadFinish() {
                        // example: how to start directions programmatically
                        /*    for (placemark in mapFragment.mapView.placemarks) {
                            if ("APPLE" == placemark.name) {
                                mapFragment.startDirections(DirectionsDestination.forPlacemarkKey(placemark.key))
                            }
                        }*/
                        // example: highlight the first four placemarks
                        /*
                        val markerList: java.util.ArrayList<com.arubanetworks.meridian.maps.Marker> = java.util.ArrayList()
                        var index = 0
                        val mapView: MapView = mapFragment.getMapView()
                        if (mapView.getAllMarkers() != null) {
                            for (marker in mapView.getAllMarkers()) {
                                markerList.add(marker)
                                index++
                                if (index >= 4) {
                                    val highlightedMarkers: HighlightedMarkers = HighlightedMarkers.Builder(markerList).build()
                                    mapView.commitTransaction(com.arubanetworks.meridian.maps.Transaction.Builder().setAnimationDuration(500).addMarker(highlightedMarkers).build())
                                    break
                                }
                            }
                        }
                        */
                        // example: change color of first placemark marker
                        /*
                        for (marker in mapFragment.mapView.allMarkers) {
                            if (marker is FlatPlacemarkMarker) {
                                (marker as FlatPlacemarkMarker).tintColor = Color.RED
                                break
                            }
                        }
                         */
                    }

                    override fun onMapRenderFinish() { }

                    override fun onMapLoadFail(tr: Throwable?) {
                        if (mapFragment.isAdded && mapFragment.activity != null) {
                            var message = getString(com.arubanetworks.meridian.R.string.mr_error_invalid_map)
                            if (tr != null) {
                                if (tr is VolleyError && tr.networkResponse != null && tr.networkResponse.statusCode == 401) {
                                    message = "HTTP 401 Error: Please verify the Editor token."
                                } else {
                                    tr.localizedMessage?.let { message = it }
                                }
                            }
                            AlertDialog.Builder(mapFragment.requireActivity())
                                    .setTitle(getString(com.arubanetworks.meridian.R.string.mr_error_title))
                                    .setMessage(message)
                                    .setIcon(android.R.drawable.ic_dialog_alert)
                                    .setPositiveButton(com.arubanetworks.meridian.R.string.mr_ok, null)
                                    .show()
                        }
                    }

                    override fun onMapTransformChange(transform: Matrix) { }

                    override fun onLocationUpdated(location: MeridianLocation) { }

                    override fun onOrientationUpdated(orientation: MeridianOrientation) { }
                })
                mapFragment.setMapClickEventListener(object : MapView.MapClickEventListener {
                    override fun onLocationButtonClick(): Boolean {
                        // example of how to override the behavior of the location button
                        val mapView = mapFragment.mapView
                        val location = mapView.userLocation
                        if (location != null) {
                            mapView.updateForLocation(location)

                        } else {
                            LocationRequest.requestCurrentLocation(applicationContext, APP_KEY, object : LocationRequest.LocationRequestListener {
                                override fun onResult(location: MeridianLocation) {
                                    mapView.updateForLocation(location)
                                }

                                override fun onError(location: LocationRequest.ErrorType) {
                                    // handle the error
                                }
                            })
                        }
                        return true
                    }
                    override fun onOverviewButtonClick(): Boolean {
                        return false
                    }
                    override fun onLongPress(): Boolean {
                        return false
                    }
                    override fun onMapClick(): Boolean {
                        return false
                    }
                })
                fragment = mapFragment
            }
            1 -> fragment = NearbyFragment.newInstance(APP_KEY)
            2 -> fragment = SearchFragment.newInstance(APP_KEY)
            3 -> fragment = CustomMarkerFragment.newInstance(MAP_KEY)
            4 -> fragment = TriggersFragment.newInstance(APP_KEY)
            5 -> fragment = ScrollingFragment.newInstance(MAP_KEY)
            6 -> fragment = LocationFragment.newInstance(APP_KEY)
            7 -> fragment = LocationSharingFragment.newInstance()
            8 -> fragment = SingleMarkerIDFragment.newInstance(APP_KEY, PLACEMARK_UID, null)
            9 -> {
                fragment = if (!Strings.isNullOrEmpty(Application.TAG_MAC)) {
                    TagsFragment.newInstance(MAP_KEY, Application.TAG_MAC)
                } else {
                    TagsFragment.newInstance(MAP_KEY)
                }
            }
            10 -> fragment = LocationSharingMapFragment.newInstance(MAP_KEY)
            11 -> fragment = OverlayMarkerFragment.newInstance(MAP_KEY)
            12 -> fragment = MapViewFragment()
            13 -> {
                fragment = GoogleMapsFragment()
                // verify that the google maps API key has been set in the manifest
                if (!useGoogleMaps()) {
                    AlertDialog.Builder(this)
                            .setMessage("You need to provide a valid Google Maps API key in the manifest.")
                            .setPositiveButton("OK", null)
                            .show()
                    return
                }
            }
            14 -> {
                // placemark deeplink intent sample
                val urlScheme: String = getString(R.string.url_scheme)

                val packageManager: PackageManager = applicationContext.packageManager

                val intent: Intent? =
                    packageManager.getLaunchIntentForPackage(applicationContext.packageName)

                //placemarks deeplink uri format: com.arubanetworks.meridian-samples://12345/placemarks/34567_67890
                val uri: String =
                    urlScheme + "://" + APP_KEY.id + "/placemarks/" + MAP_KEY.id + "_" + PLACEMARK_UID

                if (intent != null) {
                    intent.action = Intent.ACTION_VIEW
                    intent.data = Uri.parse(uri)
                    applicationContext.startActivity(intent)
                }
                return
            }
            else -> return
        }

        supportFragmentManager
                .beginTransaction()
                .replace(R.id.container, fragment)
                .commit()

        title = resources.getStringArray(R.array.section_titles)[position]
        drawerList?.let { drawerLayout?.closeDrawer(it) }
    }

    override fun setTitle(title: CharSequence) {
        val actionBar = supportActionBar
        actionBarTitle = title
        actionBar?.title = actionBarTitle
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            // now request background permissions, should use a fancy UI for this
            if (permissions[0] == Manifest.permission.ACCESS_FINE_LOCATION && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                val permissions2 = arrayOf(Manifest.permission.ACCESS_BACKGROUND_LOCATION)
                ActivityCompat.requestPermissions(this, permissions2, 1)
                return
            }
        }
        selectItem(0)
    }

    /**
     * When using the ActionBarDrawerToggle, you must call it during
     * onPostCreate() and onConfigurationChanged()...
     */
    override fun onPostCreate(savedInstanceState: Bundle?) {
        super.onPostCreate(savedInstanceState)
        // Sync the toggle state after onRestoreInstanceState has occurred.
        drawerToggle?.syncState()
    }

    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        // Pass any configuration changes to the drawer toggles
        drawerToggle?.onConfigurationChanged(newConfig)
    }
}
